"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const anymatch_1 = tslib_1.__importDefault(require("anymatch"));
const fs_1 = tslib_1.__importDefault(require("fs"));
const settings_1 = tslib_1.__importDefault(require("../settings"));
const parser_1 = tslib_1.__importDefault(require("../parser"));
const nodeIsImportDeclaration = (node) => node.type === "ImportDeclaration";
const nodeIsImportSpecifier = (node) => node.type === "ImportSpecifier";
const nodeIsImportDefaultSpecifier = (node) => node.type === "ImportDefaultSpecifier";
const nodeIsImportNamespaceSpecifier = (node) => node.type === "ImportNamespaceSpecifier";
const nodeIsMemberExpression = (node) => node.type === "MemberExpression";
const rule = {
    meta: {
        docs: {
            description: "",
            recommended: true,
        },
        messages: {
            undefinedClassName: "{{ className }} does not exist in {{ baseFilename }}",
        },
        schema: [
            {
                type: "object",
                properties: {
                    baseDir: {
                        description: "Base directory for resolving 'absolute' imports",
                        type: "string",
                    },
                    camelCase: {
                        description: "How classes are exported. See the documentation for css-loader: https://github.com/webpack-contrib/css-loader/tree/v2.1.1#camelcase",
                        oneOf: [
                            { type: "boolean" },
                            {
                                enum: ["dashes", "dashesOnly", "only"],
                            },
                        ],
                    },
                    defaultScope: {
                        description: "The default scope of classes that are not explicitly scoped: local, global, or pure... I'm not sure what pure is; it's not documented.",
                        enum: ["local", "global", "pure"],
                    },
                    include: {
                        description: "Anymatch describing what files to parse.",
                        type: "any",
                    },
                    exclude: {
                        description: "Anymatch describing what files to exclude from parsing.",
                        type: "any",
                    },
                },
                additionalProperties: false,
            },
        ],
    },
    create: (context) => {
        const settings = new settings_1.default(context);
        const parser = new parser_1.default(settings);
        const specifierToClasses = {};
        return {
            ImportDeclaration: node => {
                if (!nodeIsImportDeclaration(node) ||
                    typeof node.source.value !== "string") {
                    return;
                }
                const baseFilename = node.source.value;
                const filename = settings.resolveFile(baseFilename);
                if (!anymatch_1.default(settings.include, filename) ||
                    (settings.exclude && anymatch_1.default(settings.exclude, filename)) ||
                    !fs_1.default.existsSync(filename)) {
                    return;
                }
                const importsToCheck = [];
                let specifier = null;
                node.specifiers.forEach(spec => {
                    if (nodeIsImportSpecifier(spec)) {
                        importsToCheck.push(spec);
                    }
                    else if (nodeIsImportDefaultSpecifier(spec) ||
                        nodeIsImportNamespaceSpecifier(node)) {
                        specifier = spec.local.name;
                    }
                });
                const classes = parser.parse(filename);
                if (specifier) {
                    specifierToClasses[specifier] = {
                        baseFilename,
                        classes,
                    };
                }
                importsToCheck.forEach(node => {
                    const className = node.imported.name;
                    if (!classes.has(className)) {
                        context.report({
                            messageId: "undefinedClassName",
                            node,
                            data: {
                                baseFilename,
                                className,
                            },
                        });
                    }
                });
            },
            MemberExpression: node => {
                if (!nodeIsMemberExpression(node)) {
                    return;
                }
                const objectName = node.object.name;
                if (specifierToClasses[objectName] === undefined) {
                    return;
                }
                const { baseFilename, classes } = specifierToClasses[objectName];
                const className = node.computed
                    ? node.property.value
                    : node.property.name;
                if (typeof className === "string" &&
                    !classes.has(className) &&
                    !className.startsWith("_")) {
                    context.report({
                        messageId: "undefinedClassName",
                        node,
                        data: {
                            baseFilename,
                            className,
                        },
                    });
                }
            },
        };
    },
};
exports.default = rule;
